#include "stdafx.h"
#include "FidoLib.h"

#include <string.h>
#include <iostream>

#ifdef __LINUX__
#include "lnxadapt.h"
#endif

extern char logfilename[256];

CFidoLib::CFidoLib(void)
{
	status=0;
	for(int i=0;i<FIDO_LIB_MAX_PACKETS_NUM;i++)
		packet[i]=NULL;
	packets_num=0;

	pkt_data=NULL;
	pkt_data_len=0;
	pkt_data_load=0;
}

CFidoLib::~CFidoLib(void)
{
	Free();
}

bool CFidoLib::OpenPackets(char* FileName)
{
	if(!status)
	{
		FILE *fl;
		fl=fopen(FileName,"rb");

		if(!fl)
			return false;

		Free();

		pkt_data_len=filesize(fl);
		pkt_data=new unsigned char [pkt_data_len];
		fread((void*)pkt_data,1,pkt_data_len,fl);
		pkt_data_load=1;

		fclose(fl);
		status=1;

		long pkt_pos;

		pkt_pos=0x3A;

		while(pkt_pos<(pkt_data_len-19))
			pkt_pos=GetNextPacket(pkt_pos);

		delete[]pkt_data;
		pkt_data=NULL;
		pkt_data_len=0;
		pkt_data_load=0;

        return true;
	}
	else
        return false;
}

void CFidoLib::Free(void)
{
	status=0;
	for(int i=0;i<FIDO_LIB_MAX_PACKETS_NUM;i++)
		if(packet[i])
		{
			if(packet[i]->text)
			{
				delete[]packet[i]->text;
				packet[i]->text=NULL;
				packet[i]->text_len=0;
			};
			delete[]packet[i];
			packet[i]=NULL;
		};
	packets_num=0;

	if(pkt_data)
	{
		delete[]pkt_data;
		pkt_data_len=0;
		pkt_data_load=0;
	};
}

void CFidoLib::Init(void)
{
	if(status)
		Free();
	status=1;
}

long CFidoLib::GetNextPacket(long pos)
{
	if(status&&pkt_data_load)
	{
		bool msg_pass;

		msg_pass=true;

		pos+=14;

		SPacket *pkt;
		pkt=new SPacket;

		memset((void*)pkt,0,sizeof(SPacket));
		pkt->text=NULL;

		char buf[FIDO_LIB_MAX_BUF_LEN];
		long buf_len;

		/*  y ASCIIz :              */

		buf_len=strlen((char*)pkt_data+pos);
		if(buf_len<FIDO_LIB_MAX_DATA_LEN && msg_pass)
		{
			memset((void*)pkt->data,0,FIDO_LIB_MAX_DATA_LEN);
			strcpy(pkt->data,(char*)pkt_data+pos);
		}
		else
			msg_pass=0;
		pos+=buf_len+1;

		buf_len=strlen((char*)pkt_data+pos);
		if(buf_len<FIDO_LIB_MAX_NAME_LEN && msg_pass)
		{
			memset((void*)pkt->to,0,FIDO_LIB_MAX_NAME_LEN);
			strcpy(pkt->to,(char*)pkt_data+pos);
		}
		else
			msg_pass=0;
		pos+=buf_len+1;

		buf_len=strlen((char*)pkt_data+pos);
		if(buf_len<FIDO_LIB_MAX_NAME_LEN && msg_pass)
		{
			memset((void*)pkt->from,0,FIDO_LIB_MAX_NAME_LEN);
			strcpy(pkt->from,(char*)pkt_data+pos);
		}
		else
			msg_pass=0;
		pos+=buf_len+1;

		buf_len=strlen((char*)pkt_data+pos);
		if(buf_len<FIDO_LIB_MAX_SUBJ_LEN && msg_pass)
		{
			memset((void*)pkt->subj,0,FIDO_LIB_MAX_NAME_LEN);
			strcpy(pkt->subj,(char*)pkt_data+pos);
		}
		else
			msg_pass=0;
		pos+=buf_len+1;

		buf_len=strlen((char*)pkt_data+pos);
		if(buf_len<FIDO_LIB_MAX_TEXT_LEN && msg_pass)
		{
			pkt->text=new char[buf_len+1];

			if(pkt->text==NULL)
			{
				msg_pass=0;
			}
			else
			{
				pkt->text_len=buf_len+1;
				memset((void*)pkt->text,0,pkt->text_len);
				strcpy(pkt->text,(char*)pkt_data+pos);
			};
		}
		else
			msg_pass=0;
		pos+=buf_len+1;

		if(msg_pass)
		{
			ParsePacket(pkt);
			packet[packets_num++]=pkt;
		}
		else
		{
			delete pkt;
            add_to_log(logfilename,".   . ().");
		};

		return pos;
	}
	else
        return 0x00FFFFFF;
}

bool CFidoLib::ParsePacket(SPacket* pkt)
{
	OemToChar(pkt->from,pkt->from);
	OemToChar(pkt->to,pkt->to);
	OemToChar(pkt->subj,pkt->subj);
	OemToChar(pkt->text,pkt->text);
	OemToChar(pkt->data,pkt->data);

	char *line,*parsed,*src;
	long line_pos,line_len,parsed_pos,parsed_len,src_pos,src_len;

	bool area_f,addr_f;

	area_f=false;
	addr_f=false;

	line=new char [FIDO_LIB_MAX_BUF_LEN];line_pos=0;line_len=0;parsed_pos=0;
	parsed=new char [FIDO_LIB_MAX_BUF_LEN];parsed_len=0;parsed_pos=0;

	src_pos=0;
	src=pkt->text;
	src_len=pkt->text_len-1;

	while(src_pos<src_len)
	{
		line_pos=0;
		line_len=GetLine(src,src_pos,line);
		src_pos+=line_len+1;

		if(*line==1)
		{
			if(strncmp("MSGID:",line+1,6)==0)
			{
				pkt->msgid=GetID(line+line_len-8);
		if(pkt->msgid==0)
			long tt=23;
				continue;
			};

			if(strncmp("REPLY:",line+1,6)==0)
			{
				pkt->reply=GetID(line+line_len-8);
				continue;
			};

			continue;
		}

		if(!area_f)
			if(strncmp("AREA:",line,5)==0)
			{
				memcpy((void*)pkt->area,line+5,line_len-5<FIDO_LIB_MAX_AREA_LEN?line_len-5:FIDO_LIB_MAX_AREA_LEN-1);
				area_f=true;
				continue;
			};

		if(!addr_f)
			if(strncmp(" * Origin:",line,10)==0)
			{
				int p,i1,i2;

				p=line_len;
				while(*(line+p)!='(')
					p--;

				i1=p+1;

				while(*(line+p)!=')' && *(line+p)!='@' && *(line+p))
					p++;

				i2=p;

				memcpy((void*)pkt->addr,line+i1,i2-i1);

				addr_f=true;
//				continue;
			};

		if(strncmp("SEEN-BY: ",line,9)==0)
			continue;

		parsed_pos+=AddLine(parsed,line,parsed_pos);
	};

	if(!area_f)
		strcpy(pkt->area,"-NETMAIL-");

	parsed[parsed_pos]=0;
	parsed_len=parsed_pos;

	src=NULL;

	delete[](pkt->text);

	pkt->text=new char [strlen(parsed+1)+1];
	memcpy((void*)pkt->text,parsed+1,parsed_len);
	pkt->text_len=strlen(pkt->text)+1;

	delete[]line;
	return true;
}

long CFidoLib::GetLine(char *src,long pos,char *dst)
{
	int len;

	len=0;
	memset((void*)dst,0,FIDO_LIB_MAX_BUF_LEN);

	while(*(src+pos) && *(src+pos)!=0x0D && len<FIDO_LIB_MAX_BUF_LEN)
	{
		dst[len]=src[pos];
		pos++;
		len++;
	};
	dst[len]=0;

	return len;
}

long CFidoLib::AddLine(char* dst,char* line, long pos)
{
//	if(pos>0)
//	{
		dst[pos]=0x0A;
		pos++;
//	};
	strcpy(dst+pos,line);
	return strlen(line)+1;
}

long CFidoLib::GetPacketsNum(void)
{
	return packets_num;
}

SPacket* CFidoLib::GetPackect(long index)
{
	if(index>=0 && index<packets_num)
		return packet[index];
	else
		return NULL;
}

UInt16 flRevert16(UInt16 value)
{
	return ((value&0xFF)<<8)&0xFF00|((value>>8)&0xFF);
}

long CFidoLib::pktToBuf(char* buf,int index)
{
	SPacket *sp;

	sp=GetPackect(index);
	int pos;

	if(sp)
	{
		pos=4+4+FIDO_LIB_MAX_AREA_LEN+20+FIDO_LIB_MAX_NAME_LEN*2+16+FIDO_LIB_MAX_SUBJ_LEN+2+2;
		sp->text_len=flRevert16(sp->text_len);
		memcpy((void*)buf,(void*)sp,pos);
		sp->text_len=flRevert16(sp->text_len);
		memcpy((void*)(buf+pos),sp->text,sp->text_len);pos+=sp->text_len;
		return pos;
	};

	return 0;
}

unsigned long CFidoLib::GetID(char* text)
{
	unsigned long sm,t,i;

	sm=0;

	for(i=0;i<8;i++)
	{
		if(!text[i])
			return 0;

		t=text[i];

		if(t>='0' && t<='9')
		{
			t=t-'0';
			sm+=t<<(4*(7-i));
			continue;
		}

		if(t>='a' && t<='f')
		{
			t=t-'a'+10;
			sm+=t<<(4*(7-i));
			continue;
		}

		if(t>='A' && t<='F')
		{
			t=t-'A'+10;
			sm+=t<<(4*(7-i));
			continue;
		}
	};
	return sm;
}
