#pragma once

typedef unsigned short UInt16;

#define FIDO_LIB_MAX_DATA_LEN		20
#define FIDO_LIB_MAX_NAME_LEN		36
#define FIDO_LIB_MAX_SUBJ_LEN		72
#define FIDO_LIB_MAX_AREA_LEN		40
#define FIDO_LIB_MAX_TEXT_LEN		(1024*32)

#define FIDO_LIB_MAX_PACKETS_NUM	32768

#define FIDO_LIB_MAX_BUF_LEN		65536


/* ------  p   p .MSG  ----------------------------------- */
/*     [+] - y  Pkt      */
#define Msg_Private  0x0001       /*+Pvt -                             */
#define Msg_Crash    0x0002       /*+Cra - pp                */
#define Msg_Read     0x0004       /* Rec - p y             */
#define Msg_Sent     0x0008       /* Snt -                            */
#define Msg_File     0x0010       /*+Att -                           */
#define Msg_Transit  0x0020       /* Trs - p                        */
#define Msg_Orphan   0x0040       /* Orp - y  yy          */
#define Msg_Kill     0x0080       /* K/s -                */
#define Msg_Local    0x0100       /* Loc -    y            */
#define Msg_Hold     0x0200       /* Hld -   y y       */
#define Msg_FReq     0x0800       /* FRq - p                       */
#define Msg_RReq     0x1000       /*+RRq - p p p       */
#define Msg_IsRR     0x2000       /*+RRc - p p              */
#define Msg_AReq     0x4000       /*+ARq - Audit request ???                 */
#define Msg_FUpsReq  0x8000       /* URq - p             */

   /* pyyp                  */
#pragma pack(1)                   /*  y pyyp            */
struct FD_Message
{
	char          FromUser[36];     /*  p  From:                  */
	char          ToUser[36];       /*  y   To:                    */
	char          Subject[72];      /*  Subj:                              */
	char          Date[20];         /*    [dd mmm yy hh:mm:ss] */
	UInt16      TimesRead;        /* Number of times message read            */
	UInt16      DestNode;         /* Hp  y   To:             */
	UInt16      OrigNode;         /* Hp  p  From:           */
	UInt16      Cost;             /*  p               */
	UInt16      OrigNet;          /* Hp  p  From:           */
	UInt16      DestNet;          /* Hp  y   To:             */
	unsigned long DateWritten;      /*           (DOS p)      */
	unsigned long DateArrived;      /*  p/   (DOS p)      */
	UInt16      ReplyTo;          /*   py       */
	UInt16      Attr;             /*  py                         */
	UInt16      ReplyNext;        /*   y            */
};
#pragma pack()                    /*  pyyp  y         */

/* ------     p .PKT  -------------------------------- */

/*  .PKT  y y                                      */

#define PktVersion  2             /* y   p 2     */
#define CW_2Plus   0x0001         /*Support for Type 2+ bundles              */
#define CW_N2Plus  0x0100         /*Validation for CW_2PLUS                  */

/* pyyp   .PKT												*/
#pragma pack(1)						/*  y pyyp				*/
struct FD_Packet
{
	UInt16      OrigNode;         /* Hp  p  From:			*/
	UInt16      DestNode;         /* Hp  y   To:				*/
	UInt16      Year;             /*   ()						*/
	UInt16      Month;            /*   ()                   */
	UInt16      Day;              /*   ()                    */
	UInt16      Hour;             /* p  ()                   */
	UInt16      Minute;           /* p  (y)                 */
	UInt16      Second;           /* p  (y)                */
	UInt16      Rate;             /* Baudrate = 0  ( y)         */
	UInt16      Version;          /* p p  = 2               */
	UInt16      OrigNet;          /* Hp  p  From:           */
	UInt16      DestNet;          /* Hp  y   To:             */
	unsigned char PCodeLo;          /*   pp -low            */
	unsigned char PRevMajor;        /* p  ppp           */
	unsigned char Password[8];      /* p    ( <nul>)            */
	UInt16      QMOrigZone;       /* = OrigZone  ( QMail)                 */
	UInt16      QMDestZone;       /* = DestZone  (QMail)                     */
	UInt16      AuxNet;           /* Auxillary net = 0  ( y)    */
	UInt16      CWValidate;       /* CapWord validation = 0x0100             */
	unsigned char PCodeHi;          /*   pp -high           */
	unsigned char PRevMinor;        /*   pp          */
	UInt16      CWCapWord;        /* Capabilities = 0x0001                   */
	UInt16      OrigZone;         /* Hp  p  From:           */
	UInt16      DestZone;         /* Hp  y   To:             */
	UInt16      OrigPoint;        /* Hp  p  From:         */
	UInt16      DestPoint;        /* Hp  y   To:           */
	unsigned long LongData;         /*   p                  */
};
#pragma pack()						/*  pyyp  y         */

/*---------   y  Pkt   ---------------------------- */
#pragma pack(1)						/*  y pyyp            */
struct FD_MsgInPkt
{
	UInt16      Version;          /* p   = 2                      */
	UInt16      OrigNode;         /* Hp y p                  */
	UInt16      DestNode;         /* Hp  y                   */
	UInt16      OrigNet;          /* Hp  p                  */
	UInt16      DestNet;          /* Hp  y                   */
	UInt16      Attr;             /*                     */
	UInt16      Cost;             /*  p               */
									/*  y ASCIIz :              */
									/* AsciiDate - p   p  */
									/* ToUser    -  y  To:         */
									/* FromUser  -  p From:       */
									/* Subject   -  Subj:            */
									/* MsgText   -        */
};
#pragma pack()	                    /*  pyyp  y         */


struct SPacket
{
	unsigned long msgid,reply;
	char area[FIDO_LIB_MAX_AREA_LEN];
	char data[FIDO_LIB_MAX_DATA_LEN];
	char from[FIDO_LIB_MAX_NAME_LEN];
	char addr[16];
	char to[FIDO_LIB_MAX_NAME_LEN];
	char subj[FIDO_LIB_MAX_SUBJ_LEN];
	UInt16 attr;
	UInt16 text_len;
	char *text;
};

class CFidoLib
{
private:
	long status;
	SPacket *packet[FIDO_LIB_MAX_PACKETS_NUM];
	long packets_num;

    long pkt_data_load;
	unsigned char *pkt_data;
	long pkt_data_len;

	long GetNextPacket(long pos);
	bool ParsePacket(SPacket* pkt);
	long GetLine(char *src,long pos,char *dst);
	long AddLine(char *dst,char *line, long pos);

protected:
	unsigned long GetID(char* text);

public:
	CFidoLib(void);
	~CFidoLib(void);
	bool OpenPackets(char* FileName);
	void Free(void);
	void Init(void);
	long GetPacketsNum(void);
	SPacket* GetPackect(long index);
	long pktToBuf(char* buf,int index);
};
