#include "stdafx.h"
#include "libPalmDB.h"
#include <string.h>
#include <time.h>
#include <iostream>

CPalmDB::CPalmDB(void)
{
	status=PDB_STATUS_NULL;
	records_num=0;
	for(unsigned short i=0;i<PDB_MAX_RECORD_NUM;i++)
	{
		record[i]=NULL;
		record_len[i]=0;
	};
};

CPalmDB::~CPalmDB(void)
{
	FreePDB();
};

bool CPalmDB::CreatePDB(char* db_name, unsigned short attributes, unsigned short version, char* type, char* creator)
{
	if(status!=PDB_STATUS_NULL)
		return false;

	memset((void*)&m_pdb_header,0,sizeof(m_pdb_header));
	strcpy((char*)m_pdb_header.name,db_name);
	m_pdb_header.attributes=Revert16(attributes);
	m_pdb_header.version=Revert16(version);
	m_pdb_header.creationDate=Revert32(GetTime()+TIME_DELTA);
	m_pdb_header.modificationDate=Revert32(GetTime()+TIME_DELTA);
	m_pdb_header.lastBackupDate=0;
	m_pdb_header.modificationNumber=0;
	m_pdb_header.appInfoID=0;
	m_pdb_header.sortInfoID=0;
	m_pdb_header.type=type[0]+type[1]*0x100+type[2]*0x10000+type[3]*0x1000000;
	m_pdb_header.creator=creator[0]+creator[1]*0x100+creator[2]*0x10000+creator[3]*0x1000000;
	m_pdb_header.uniqueIDSeed=0;

	m_pdb_header.recordList.nextRecordListID=0;
	m_pdb_header.recordList.numRecords=0;
	m_pdb_header.recordList.firstEntry=0;

	status=PDB_STATUS_OPEN;
}

UInt16 CPalmDB::Revert16(UInt16 value)
{
	return ((value&0xFF)<<8)&0xFF00|((value>>8)&0xFF);
}

UInt32 CPalmDB::Revert32(UInt32 value)
{
/*	UInt32 vh,vl;

	vh=(value&0xFFFF0000ul)>>16;
	vl=value&0xFFFFul;

	vh=Revert16(vh);
	vl=Revert16(vl);

	return (vl<<16)|vh;
*/
	return (((UInt32)Revert16(value&0xFFFFu))<<16)|((UInt32)Revert16((value>>16)&0xFFFFu));
}

UInt32 CPalmDB::GetTime(void)
{
	return (UInt32)time(NULL);
}

signed long CPalmDB::GetFirstFreeRecord(void)
{
	if(status==PDB_STATUS_NULL)
		return -1;

	for(unsigned short i=0;i<PDB_MAX_RECORD_NUM;i++)
		if(record[i]==NULL && record_len[i]==0)
			return i;

	return -1;
}

unsigned short CPalmDB::GetRecordsNum(void)
{
	if(status==PDB_STATUS_NULL)
		return 0;

	unsigned short num;

	num=0;
	for(unsigned short i=0;i<PDB_MAX_RECORD_NUM;i++)
		if(record[i]!=NULL && record_len[i]!=0)
			num++;

	return num;
}

bool CPalmDB::SetRecord(unsigned short index, unsigned char* data,unsigned short data_len)
{
	if(status==PDB_STATUS_NULL)
		return false;

	if(record[index])
		FreeRecord(index);

	record[index]=new unsigned char [data_len];

	if(!record[index])
		return false;

	memcpy(record[index],data,data_len);
	record_len[index]=data_len;

	return true;
}

bool CPalmDB::FreeRecord(unsigned short index)
{
	if(status==PDB_STATUS_OPEN)
		return false;

	if(record[index] &&	record_len[index])
	{
		delete[] record[index];
		record[index]=NULL;
		record_len[index]=0;
		records_num--;
		return true;
	};
	return false;
};

bool CPalmDB::AddRecord(unsigned char* data, unsigned short data_len)
{
	if(status==PDB_STATUS_NULL)
		return false;

	signed long free_index;

	free_index=GetFirstFreeRecord();

	if(free_index!=-1)
		if(SetRecord(free_index,data,data_len))
		{
			records_num++;
			return true;
        }
	else
        return false;
}

bool CPalmDB::SavePDB(char* pdbFileName)
{
	if(status==PDB_STATUS_NULL)
		return false;

	FILE *fl;

	fl=fopen(pdbFileName,"wb");

	if(!fl)
		return false;

	m_pdb_header.recordList.numRecords=Revert16(records_num);

	fwrite((void*)&m_pdb_header,1,sizeof(m_pdb_header)-2,fl);

	UInt32 rec_position;
	RecordEntryType ret;
	unsigned short i;

	rec_position=0x4E+(records_num*8)+2;

	for(i=0;i<records_num;i++)
	{
		ret.attributes=PDB_RECORD_ATTR;
		ret.localChunkID=Revert32(rec_position);
		ret.uniqueID[0]=((PDB_UNIQUE_ID+i)&0xFF0000)>>16;
		ret.uniqueID[1]=((PDB_UNIQUE_ID+i)&0xFF00)>>8;
		ret.uniqueID[2]=((PDB_UNIQUE_ID+i)&0xFF);
		fwrite((void*)&ret,1,sizeof(ret),fl);
		rec_position+=record_len[i];
	};

	fputc(0x00,fl);
	fputc(0x00,fl);

	for(i=0;i<records_num;i++)
		fwrite((void*)record[i],1,record_len[i],fl);

	fclose(fl);

	return true;
}

bool CPalmDB::OpenPDB(char* pdbFileName)
{
	if(status!=PDB_STATUS_NULL)
		return false;

	FILE *fl;

	fl=fopen(pdbFileName,"rb");

	if(!fl)
		return false;

	FreePDB();
	status=PDB_STATUS_OPEN;

	fread((void*)&m_pdb_header,1,sizeof(m_pdb_header)-2,fl);

	if(Revert16(m_pdb_header.recordList.numRecords)!=0)
	{
		RecordEntryType ret;
		unsigned short i;

		UInt32 stpos;

		stpos=0x4E+(Revert16(m_pdb_header.recordList.numRecords)*8)+2;

		for(i=0;i<Revert16(m_pdb_header.recordList.numRecords);i++)
		{
			fread((void*)&ret,1,sizeof(ret),fl);
			if(i>0)
			{
				record_len[i-1]=Revert32(ret.localChunkID)-stpos;
				stpos=Revert32(ret.localChunkID);
			};
		};

		record_len[i-1]=filesize(fl)-stpos;

		fgetc(fl);
		fgetc(fl);

		unsigned char tbuf[PDB_MAX_RECORD_LEN];
		unsigned short tbuf_len;

		for(i=0;i<Revert16(m_pdb_header.recordList.numRecords);i++)
		{
			tbuf_len=record_len[i];
			fread(tbuf,1,record_len[i],fl);
			record_len[i]=0;
			AddRecord(tbuf,tbuf_len);
		};
	}
	else
	{
		fgetc(fl);
		fgetc(fl);
	};

	fclose(fl);

	return true;
}

void CPalmDB::FreePDB(void)
{
	if(status!=PDB_STATUS_NULL)
	{
		for(unsigned short i=0;i<PDB_MAX_RECORD_NUM;i++)
			if(record[i])
			{
				delete[]record[i];
				record[i]=NULL;
				record_len[i]=0;
			};
		records_num=0;
		status=PDB_STATUS_NULL;
	};
}

bool CPalmDB::SortByStr(unsigned short StartPosition)
{
	if(status==PDB_STATUS_NULL)
		return false;

	if(StartPosition>48);

	unsigned short i,j,fminpos,t;
	unsigned char* fminstr;

	for(i=0;i<records_num-1;i++)
	{
		fminpos=i;
		fminstr=record[i]+StartPosition;

		for(j=i+1;j<records_num;j++)
			if(strcmp((char*)fminstr,(char*)(record[j]+StartPosition))>0)
			{
				fminpos=j;
				fminstr=record[j]+StartPosition;
			};
		
        fminstr=record[i];
		record[i]=record[fminpos];
		record[fminpos]=fminstr;

		t=record_len[i];
		record_len[i]=record_len[fminpos];
		record_len[fminpos]=t;
	};
	return true;
};
