#pragma once
typedef unsigned char	UInt8;
typedef unsigned short	UInt16;
typedef unsigned long	UInt32;
typedef unsigned long	LocalID;
#define dmDBNameLength	32

#define TIME_DELTA				((UInt32)2082844800)
#define PDB_MAX_RECORD_NUM		(unsigned short)(0xFFFFul)
#define PDB_MAX_RECORD_LEN		(unsigned short)(0xFFFFul)
#define PDB_UNIQUE_ID			(((((UInt32)('K'))<<16)|(((UInt32)('S'))<<8))&0xFFFF00ul)
#define PDB_RECORD_ATTR			(0x40)

//Palm File Format Specification
/************************************************************
* Structure of a Record entry
*************************************************************/
struct RecordEntryType
{
	LocalID localChunkID;			// local chunkID of a record
	UInt8 attributes;				// record attributes;
	UInt8 uniqueID[3];				// unique ID of record; should not be 0 for a legal record.
};

/************************************************************
* Structure of a Resource entry
*************************************************************/
struct RsrcEntryType
{
	UInt32 type;					// resource type
	UInt16 id;						// resource id
	LocalID localChunkID;			// resource data chunk ID
};

/************************************************************
* Structure of a record list extension. This is used if all
* the database record/resource entries of a database can?t fit into
* the database header.
*************************************************************/
struct RecordListType
{
	LocalID nextRecordListID;		// local chunkID of next list
	UInt16 numRecords;				// number of records in this list
	UInt16 firstEntry;				// array of Record/Rsrc entries starts here
};

/************************************************************
* Structure of a Database Header
*************************************************************/
struct DatabaseHdrType
{
	UInt8 name[dmDBNameLength];		// name of database
	UInt16 attributes;				// database attributes
	UInt16 version;					// version of database
	UInt32 creationDate;			// creation date of database
	UInt32 modificationDate;		// latest modification date
	UInt32 lastBackupDate;			// latest backup date
	UInt32 modificationNumber;		// modification number of database
	LocalID appInfoID;				// application specific info
	LocalID sortInfoID;				// app specific sorting info
	UInt32 type;					// database type
	UInt32 creator;					// database creator
	UInt32 uniqueIDSeed;			// used to generate unique IDs.
									//Note that only the low order
									//3 bytes of this is used (in
									//RecordEntryType.uniqueID).
									//We are keeping 4 bytes for
									//alignment purposes.
	RecordListType recordList;		// first record list
};

#define PDB_STATUS_NULL		0
#define PDB_STATUS_OPEN		1

class CPalmDB
{
private:
	long status;

	DatabaseHdrType m_pdb_header;
	unsigned char *record[PDB_MAX_RECORD_NUM];
	unsigned short record_len[PDB_MAX_RECORD_NUM];
	unsigned short records_num;
	
	UInt16 Revert16(UInt16 value);
	UInt32 Revert32(UInt32 value);
	UInt32 GetTime(void);
	signed long GetFirstFreeRecord(void);

public:
	CPalmDB(void);
	~CPalmDB(void);
	bool CreatePDB(char* db_name, unsigned short attributes, unsigned short version, char* type, char* creator);
	unsigned short GetRecordsNum(void);
	bool SetRecord(unsigned short index, unsigned char* data,unsigned short data_len);
	bool FreeRecord(unsigned short index);
	bool AddRecord(unsigned char* data, unsigned short data_len);
	bool SavePDB(char* pdbFileName);
	bool OpenPDB(char* pdbFileName);
	void FreePDB(void);
	bool SortByStr(unsigned short StartPosition);
};
